import keras.layers as kl
import keras.models as km
import keras.optimizers as ko
import keras.backend as kb
import tensorflow as tf
import numpy as np

def make_model():
    input_seq = kl.Input(shape=(1000, 4))
    conv_out = kl.Conv1D(
        filters=10, kernel_size=10, padding="valid", activation="relu"
    )(input_seq)
    conv_out_flat = kl.Flatten()(conv_out)
    dense_out = kl.Dense(units=1, activation="sigmoid")(conv_out_flat)
    return km.Model(inputs=input_seq, outputs=dense_out)

def compute_loss(true_vals, pred_vals):
    return kb.mean(kb.square(true_vals - pred_vals))

model = make_model()

input_seqs = np.random.random((10, 1000, 4))
outputs = np.random.random((10, 1))

optimizer = ko.Adam(lr=0.001)

for i in range(100):
    with tf.GradientTape() as tape:
        preds = model(tf.convert_to_tensor(input_seqs), training=True)
        loss = compute_loss(tf.convert_to_tensor(outputs, dtype=tf.float32), preds)
        print(float(loss))
    grads = tape.gradient(loss, model.trainable_weights)
    optimizer.apply_gradients(zip(grads, model.trainable_weights))
