import torch
import numpy as np

class Model(torch.nn.Module):
    def __init__(self):
        super().__init__()
        self.conv = torch.nn.Conv1d(
            in_channels=4, out_channels=500, kernel_size=10, padding=0
        )
        self.dense = torch.nn.Linear(in_features=495500, out_features=1)

        self.relu = torch.nn.ReLU()
        self.sigmoid = torch.nn.Sigmoid()

    def forward(self, input_seqs):
        assert input_seqs.size()[1:] == (1000, 4)
        input_seqs = input_seqs.transpose(1, 2)

        conv_out = self.relu(self.conv(input_seqs))
        conv_out_flat = conv_out.reshape((conv_out.size(0), -1))
        dense_out = self.sigmoid(self.dense(conv_out_flat))

        return dense_out

    def loss(self, true_vals, pred_vals):
        return torch.mean(torch.pow(true_vals - pred_vals, 2))

model = Model().cuda()
model.train()
torch.set_grad_enabled(True)

input_seqs = np.random.random((1000, 1000, 4))
outputs = np.random.random((1000, 1))

optimizer = torch.optim.Adam(model.parameters(), lr=0.001)

for i in range(100):
    optimizer.zero_grad()
    
    preds = model(torch.tensor(input_seqs).cuda().float())
    loss = model.loss(torch.tensor(outputs).cuda().float(), preds)
    print(loss.item())
    grads = loss.backward()
    optimizer.step()
